;===============================================================================
;
; Description:      Convert Scite syntax files bundled with Autoit3 into  
;                   EditPlus 3 syntax and autocomplete files.
; Requirement(s):   Autoit 3.2.12+, <Array.au3>
; Author(s):        Dmitry Yudin (Lazycat)
; Version:          1.0
; Date:             16.07.2008
;
;===============================================================================
#include <Array.au3>

; Path to Scite files
$sSciAPI  = "api\au3.api"
$sSciKeys = "au3.keywords.properties"

; Path to CTL template
$sCTLTpl  = "template.ctl"

; Path to syntax files
$sFileSTX = "Autoit3.stx"
$sFileACP = "Autoit3.acp"
$sFileCTL = "Autoit3.ctl"

; Section definitions
Dim $aCTList[1][2]
Dim $aOutKeys[5][3] = [["Functions","",$aCTList],["Macros","",$aCTList],["Sendkeys","",$aCTList],["Keywords","",$aCTList],["UDF's","",$aCTList]]

Dim $sCTLString = FileRead($sCTLTpl)

$api_file = FileRead($sSciAPI)
$aDefines = StringSplit($api_file, @LF)

$key_file = FileRead($sSciKeys)
$version = StringRegExp($key_file, "(?m)^;Autoit3 Version (.+)$", 1)
If not @error Then
    $version = $version[0]
Else
    $version = "3.x"
EndIf

; Get the sendkeys keywords
$aSendKeysGroup = StringRegExp($key_file, "(?s)au3\.keywords\.sendkeys=(.+)(?:au3\.keywords|\z)", 1)
If not @error Then 
    $aSendKeys = StringRegExp($aSendKeysGroup[0], "[\w!#^\{\}+]{2,}", 3)
    If not @error Then
        For $i = 0 To UBound($aSendKeys) - 1
            $aOutKeys[2][1] &= StringUpper($aSendKeys[$i]) & "|"
            _IncArray($aOutKeys[2][2], StringUpper($aSendKeys[$i]))
        Next
    EndIf
EndIf

For $i = 1 To $aDefines[0]
    $aDefines[$i] = StringStripCR($aDefines[$i])
    $aLine = StringRegExp($aDefines[$i], "(?m)^([\w\d#@-]+)\?(\d)$", 1)
    $group = ""
    If not @error Then
        If $aLine[1] = 3 Then 
            $group = "Macros"
        ElseIf $aLine[1] = 4 Then 
            $group = "Keywords"
        Else
            ; if no group assigned, then it's directive, it's colored by "#LINECOMMENT2"
            ; due limitation (bug?) of EditPlus stx, where keyword can't start with #
            ContinueLoop
        EndIf
        For $j = 0 To 4
            If $aOutKeys[$j][0] = $group Then 
                $aOutKeys[$j][1] &= $aLine[0] & "|"
                _IncArray($aOutKeys[$j][2], $aLine[0])
            EndIf
        Next
    Else
        $aLine = StringRegExp($aDefines[$i], "(?m)^([\w\d]+)\s*\((.+?)\)\s.+$", 1)
        If not @error Then
            If StringLeft($aLine[0], 1) = "_" Then ; UDF
                $group = "UDF's"
            Else
                $group = "Functions"
            EndIf
        EndIf
        For $j = 0 To 4
            If ($aOutKeys[$j][0] = $group) AND NOT StringInStr($aOutKeys[$j][1], $aLine[0] & "|") Then 
                $aOutKeys[$j][1] &= $aLine[0] & "|"
                _IncArray($aOutKeys[$j][2], $aLine[0], $aLine[1])
            EndIf
        Next
    EndIf
Next

; Open all files
$hSTXFile = FileOpen($sFileSTX, 2)
$hACPFile = FileOpen($sFileACP, 2)
$hCTLFile = FileOpen($sFileCTL, 2)

; Write headers
FileWriteLine($hSTXFile, "#TITLE=AutoIt3")
FileWriteLine($hSTXFile, "; EditPlus keyword syntax file for AutoIt " & $version)
FileWriteLine($hSTXFile, "; Syntax file converted from Scite syntax with converter by YDY <ydy@darkhost.ru>")
FileWriteLine($hSTXFile, "; AutoIt (C)1999 - " & @YEAR & " Jonathan Bennett & Team")
FileWriteLine($hSTXFile, "")
FileWriteLine($hSTXFile, "#DELIMITER=,()[]+-*%/=""'~&|<>")
FileWriteLine($hSTXFile, "#QUOTATION1=""")
FileWriteLine($hSTXFile, "#QUOTATION2='")
FileWriteLine($hSTXFile, "#CONTINUE_QUOTE=y")
FileWriteLine($hSTXFile, "#LINECOMMENT=;")
FileWriteLine($hSTXFile, "#LINECOMMENT2=#")
FileWriteLine($hSTXFile, "#COMMENTON=#comments-start")
FileWriteLine($hSTXFile, "#COMMENTOFF=#comments-end")
FileWriteLine($hSTXFile, "#COMMENTON2=#cs")
FileWriteLine($hSTXFile, "#COMMENTOFF2=#ce")
FileWriteLine($hSTXFile, "#CASE=n")
FileWriteLine($hSTXFile, "#PREFIX1=.")
FileWriteLine($hSTXFile, "#PREFIX2=$")
FileWriteLine($hSTXFile, "#AUTOCASE=y")

FileWriteLine($hACPFile, "#TITLE=AutoIt3")
FileWriteLine($hACPFile, "EditPlus auto-completion file for AutoIt " & $version)
FileWriteLine($hACPFile, "; Syntax file converted from Scite syntax with converter by YDY <ydy@darkhost.ru>")
FileWriteLine($hACPFile, "; AutoIt (C)1999 - " & @YEAR & " Jonathan Bennett & Team")
FileWriteLine($hACPFile, "")
FileWriteLine($hACPFile, "#CASE=y")
FileWriteLine($hACPFile, "")

For $i = 0 To 4
    ; Write syntax file
    $aTemp = StringSplit(StringTrimRight($aOutKeys[$i][1], 1), "|")
    _ArraySort($aTemp, 0, 1, $aTemp[0])
    FileWriteLine($hSTXFile, "")
    FileWriteLine($hSTXFile, "#KEYWORD=" & $aOutKeys[$i][0])
    For $j = 1 To $aTemp[0]
        FileWriteLine($hSTXFile, $aTemp[$j])
    Next

    ; Write cliptext file
    $sTmpStr = ""
    $aTmp = $aOutKeys[$i][2]
    _ArraySort($aTmp, 0, 1, UBound($aTmp)-1)
    For $j = 1 To UBound($aTmp)-1
        $sTmpStr &= "#T=" & $aTmp[$j][0] & @CRLF
        If ($aOutKeys[$i][0] = "Functions") Or ($aOutKeys[$i][0] = "UDF's") Then
            $sParams = StringStripWS($aTmp[$j][1], 3)
            If StringLeft($sParams, 1) = '"' Then
                $sParams = '"^!' & StringTrimLeft($sParams, 1)
            Else
                $sParams = '^!' & $sParams
            EndIf
            $sTmpStr &= StringFormat("%s(%s)\r\n", $aTmp[$j][0], $sParams)
        EndIf
    Next
    
    ; Replace substs in the cliptext template
    $sCTLString = StringReplace($sCTLString, "%" & $aOutKeys[$i][0] & "%", $sTmpStr)

    ; Write autocomplete file
    If ($aOutKeys[$i][0] = "Functions") Or ($aOutKeys[$i][0] = "UDF's") Then
        FileWrite($hACPFile, $sTmpStr)
    EndIf
Next

FileWriteLine($hSTXFile, "#")
FileWriteLine($hACPFile, "#")
FileWrite($hCTLFile, $sCTLString)

; Close files
FileClose($hSTXFile)
FileClose($hACPFile)
FileClose($hCTLFile)

; Helper function
Func _IncArray(ByRef $aTmp, $val1, $val2 = "")
    ReDim $aTmp[UBound($aTmp)+1][2]
    $aTmp[UBound($aTmp)-1][0] = $val1
    $aTmp[UBound($aTmp)-1][1] = $val2
EndFunc